
/**
 * Un example d'utilisation de GtkTable, GtkImage et GdkPixbuf pour
 * creer une grille d'images. Deux billes de billard se deplacent
 * sur un tapis et rebondissent sur les bords du tapis. Si elles
 * entrent en collision, alors elle rebondissent dans la direction
 * opposee.
 *
 * AUTHOR: Guillaume Riviere (C) 2012
 * FILE: main.c
 */

#include <stdlib.h>

#include <gtk/gtk.h>
#include "callbacks.h"

/*================================================
 * Macro constantes
 *===============================================*/

#define NB_COLS 16
#define NB_ROWS 8

/*================================================
 * Structure de donnes pour une bille
 *===============================================*/

typedef struct bille {
  int x, y  ;  /* Position */
  int dx, dy ; /* Deplacement */
} *Bille ;


/*================================================
 * Variables globales
 *===============================================*/

GtkImage  *G_images[NB_ROWS][NB_COLS] ;
GdkPixbuf *G_pixbuf_tapis ;
GdkPixbuf *G_pixbuf_bille_blanche ;
GdkPixbuf *G_pixbuf_bille_rouge ;

int   G_idle_timeout = 0 ; /* ID de l'evenement timeout */
Bille G_bille1 ;
Bille G_bille2 ;

/*================================================
 * Prototytpes des fonctions utilitaires
 *===============================================*/

void redessiner_grille () ;
void deplacer_billes () ;
gboolean idle_function (gpointer user_data) ;
void start_animtation () ;
void stop_animtation () ;

/*================================================
 * Fonction principale
 *===============================================*/

int main (int argc, char *argv[]) {
  GtkBuilder *builder ;
  GtkWindow  *mainwindow ;
  GtkTable   *table ;
  char txt[20] ;
  int i, j ;

  gtk_init (&argc, &argv);

  /* Creer tous les widgets grace au builder GTK */
  builder = gtk_builder_new ();
  gtk_builder_add_from_file (builder, "gtk_table_example.glade", NULL);
  gtk_builder_connect_signals (builder, NULL);

  /*------------*/

  /*-- Recuperer les widgets construits par le builder GTK */
  mainwindow = GTK_WINDOW (gtk_builder_get_object (builder, "mainwindow"));
  table = GTK_TABLE (gtk_builder_get_object (builder, "table1"));

  /*-- Initialiser les widgets */

  /* Charger les images */
  G_pixbuf_bille_blanche = gdk_pixbuf_new_from_file ("images/bille_blanche.png", NULL) ;
  G_pixbuf_bille_rouge = gdk_pixbuf_new_from_file ("images/bille_rouge.png", NULL) ;
  G_pixbuf_tapis = gdk_pixbuf_new_from_file ("images/tapis.png", NULL) ;

  /* Redimensioner la grille d'agencement */
  gtk_table_resize (table, NB_COLS, NB_ROWS) ;

  /* Initialisation du tableau comme un "tapis vide" */
  for (i=0 ; i<NB_ROWS ; i++) {
    for (j=0 ; j<NB_COLS ; j++) {
      /* Creer un nouveau GtkImage */
      G_images[i][j] = (GtkImage *)gtk_image_new_from_pixbuf (G_pixbuf_tapis) ;

      /* Lui associer une etiquette avec ses coordonnees */
      sprintf (txt, "( %d ; %d )", j, i) ;
      gtk_widget_set_tooltip_text (GTK_WIDGET(G_images[i][j]), txt) ;

      /* Le placer dans la grille d'agencement prevue a cet effet */
      gtk_table_attach_defaults (table, (GtkWidget *)G_images[i][j], j, j+1, i, i+1) ;
    }
  }

  /* Creer la premiere bille */
  G_bille1 = (struct bille *) malloc (sizeof (struct bille)) ;

  /* Verifier le succes de l'allocation memoire */
  if (G_bille1 == NULL) {
    perror ("Erreur allocation memoire") ;
    exit (1) ;
  }

  /* Initialiser la bille */
  G_bille1->x = 1 ;
  G_bille1->y = 1 ;
  G_bille1->dx = 1 ;
  G_bille1->dy = 1 ;


  /* Creer la deuxieme bille */
  G_bille2 = (struct bille *) malloc (sizeof (struct bille)) ;

  /* Verifier le succes de l'allocation memoire */
  if (G_bille2 == NULL) {
    perror ("Erreur allocation memoire") ;
    exit (1) ;
  }

  /* Initialiser la bille */
  G_bille2->x = NB_ROWS - 2 ;
  G_bille2->y = NB_COLS - 2 ;
  G_bille2->dx = -1 ;
  G_bille2->dy = -1 ;

  /* Initialiser la matrice d'affichage */
  redessiner_grille () ;

  /*------------*/

  gtk_widget_show_all (GTK_WIDGET(mainwindow)); /* ATTENTION */

  /* Lancer la boucle GTK */
  gtk_main ();

  /* La boucle GTK est terminee, detruire maintenant tous les widgets */
  g_object_unref (G_OBJECT (builder));

  /* Fin normale du programme */
  return 0;
}

/*================================================
 * Fonctions utilitaires
 *===============================================*/

void redessiner_grille () {
  int i, j ;

  for (i=0 ; i<NB_ROWS ; i++) {
    for (j=0 ; j<NB_COLS ; j++) {

      /* Effacer l'image */
      gtk_image_clear(GTK_IMAGE(G_images[i][j])) ;

      if (i == G_bille1->x && j == G_bille1->y)
	gtk_image_set_from_pixbuf (GTK_IMAGE(G_images[i][j]), G_pixbuf_bille_rouge) ; /* Afficher bille rouge dans l'image (i,j) */
      else if (i == G_bille2->x && j == G_bille2->y)
	gtk_image_set_from_pixbuf (GTK_IMAGE(G_images[i][j]), G_pixbuf_bille_blanche) ; /* Afficher bille blanche dans l'image (i,j) */
      else
	gtk_image_set_from_pixbuf (GTK_IMAGE(G_images[i][j]), G_pixbuf_tapis) ; /* Afficher tapis dans l'image (i,j) */
    }
  }
}

void deplacer_billes () {

  /* Verifier que le deplacement de la bille 1 en x sera valide (sinon prendre direction opposee) */
  if (G_bille1->x + G_bille1->dx < 0 || G_bille1->x + G_bille1->dx >= NB_ROWS)
    G_bille1->dx = -G_bille1->dx ;

  /* Verifier que le deplacement de la bille 1 en y sera valide (sinon prendre direction opposee) */
  if (G_bille1->y + G_bille1->dy < 0 || G_bille1->y + G_bille1->dy >= NB_COLS)
    G_bille1->dy = -G_bille1->dy ;

  /* Verifier que le deplacement de la bille 2 en y sera valide (sinon prendre direction opposee) */
  if (G_bille2->x + G_bille2->dx < 0 || G_bille2->x + G_bille2->dx >= NB_ROWS)
    G_bille2->dx = -G_bille2->dx ;

  /* Verifier que le deplacement de la bille 2 en y sera valide (sinon prendre direction opposee) */
  if (G_bille2->y + G_bille2->dy < 0 || G_bille2->y + G_bille2->dy >= NB_COLS)
    G_bille2->dy = -G_bille2->dy ;

  /* Verifier qu'il n'y aura pas collision (sinon prendre directions opposees) */
  if (G_bille1->x + G_bille1->dx == G_bille2->x && G_bille1->y + G_bille1->dy == G_bille2->y ) {
    printf ("collision !\n") ;
    G_bille1->dx = -G_bille1->dx ;
    G_bille1->dy = -G_bille1->dy ;
    G_bille2->dx = G_bille2->dx ;
    G_bille2->dy = -G_bille2->dy ;
  }

  /* Deplacer les billes dans la direction en vigueur */
  G_bille1->x += G_bille1->dx ;
  G_bille1->y += G_bille1->dy ;
  G_bille2->x += G_bille2->dx ;
  G_bille2->y += G_bille2->dy ;
}

gboolean idle_function (gpointer user_data) {
  deplacer_billes () ;
  redessiner_grille () ;
  return TRUE ; /* should return FALSE if the source should be removed */
}

void start_animtation () {
  if (G_idle_timeout == 0) {
    /* Demander que la fonction idle_function() soit appellee toutes les 200 ms */
    G_idle_timeout = g_timeout_add (200, (GSourceFunc)idle_function, NULL) ;
  }
}

void stop_animtation () {
  if (G_idle_timeout > 0) {
    /* Demander de ne plus appeller la fonction idle_function() toutes les 200 ms */
    g_source_remove(G_idle_timeout) ;
    G_idle_timeout = 0 ;
  }
}
